@@ -25,11 +25,11 @@ class EdgeEvent(collections.namedtuple('EdgeEvent', ['edge', 'timestamp'])):
 
 
 class GPIO(object):
-    def __new__(cls, *args):
+    def __new__(cls, *args, **kwargs):
         if len(args) > 2:
-            return CdevGPIO.__new__(cls, *args)
+            return CdevGPIO.__new__(cls, *args, **kwargs)
         else:
-            return SysfsGPIO.__new__(cls, *args)
+            return SysfsGPIO.__new__(cls, *args, **kwargs)
 
     def __del__(self):
         self.close()
@@ -68,15 +68,16 @@ class GPIO(object):
         raise NotImplementedError()
 
     def poll(self, timeout=None):
-        """Poll a GPIO for the edge event configured with the .edge property.
+        """Poll a GPIO for the edge event configured with the .edge property
+        with an optional timeout.
 
         For character device GPIOs, the edge event should be consumed with
         `read_event()`. For sysfs GPIOs, the edge event should be consumed with
         `read()`.
 
-        `timeout` can be a positive number for a timeout in seconds, 0 for a
-        non-blocking poll, or negative or None for a blocking poll. Defaults to
-        blocking poll.
+        `timeout` can be a positive number for a timeout in seconds, zero for a
+        non-blocking poll, or negative or None for a blocking poll. Default is
+        a blocking poll.
 
         Args:
             timeout (int, float, None): timeout duration in seconds.
@@ -109,6 +110,70 @@ class GPIO(object):
         """
         raise NotImplementedError()
 
+    @staticmethod
+    def poll_multiple(gpios, timeout=None):
+        """Poll multiple GPIOs for the edge event configured with the .edge
+        property with an optional timeout.
+
+        For character device GPIOs, the edge event should be consumed with
+        `read_event()`. For sysfs GPIOs, the edge event should be consumed with
+        `read()`.
+
+        `timeout` can be a positive number for a timeout in seconds, zero for a
+        non-blocking poll, or negative or None for a blocking poll. Default is
+        a blocking poll.
+
+        Args:
+            gpios (list): list of GPIO objects to poll.
+            timeout (int, float, None): timeout duration in seconds.
+
+        Returns:
+            list: list of GPIO objects for which an edge event occurred.
+
+        Raises:
+            GPIOError: if an I/O or OS error occurs.
+            TypeError: if `timeout` type is not None or int.
+
+        """
+        if not isinstance(timeout, (int, float, type(None))):
+            raise TypeError("Invalid timeout type, should be integer, float, or None.")
+
+        # Setup poll
+        p = select.poll()
+
+        # Register GPIO file descriptors and build map of fd to object
+        fd_gpio_map = {}
+        for gpio in gpios:
+            if isinstance(gpio, SysfsGPIO):
+                p.register(gpio.fd, select.POLLPRI | select.POLLERR)
+            else:
+                p.register(gpio.fd, select.POLLIN | select.POLLRDNORM)
+
+            fd_gpio_map[gpio.fd] = gpio
+
+        # Scale timeout to milliseconds
+        if isinstance(timeout, (int, float)) and timeout > 0:
+            timeout *= 1000
+
+        # Poll
+        events = p.poll(timeout)
+
+        # Gather GPIOs that had edge events occur
+        results = []
+        for (fd, _) in events:
+            gpio = fd_gpio_map[fd]
+
+            results.append(gpio)
+
+            if isinstance(gpio, SysfsGPIO):
+                # Rewind for read
+                try:
+                    os.lseek(fd, 0, os.SEEK_SET)
+                except OSError as e:
+                    raise GPIOError(e.errno, "Rewinding GPIO: " + e.strerror)
+
+        return results
+
     def close(self):
         """Close the sysfs GPIO.
 
@@ -148,7 +213,18 @@ class GPIO(object):
     def name(self):
         """Get the line name of the GPIO.
 
-        his method is intended for use with character device GPIOs and always
+        This method is intended for use with character device GPIOs and always
+        returns the empty string for sysfs GPIOs.
+
+        :type: str
+        """
+        raise NotImplementedError()
+
+    @property
+    def label(self):
+        """Get the line consumer label of the GPIO.
+
+        This method is intended for use with character device GPIOs and always
         returns the empty string for sysfs GPIOs.
 
         :type: str
@@ -178,7 +254,7 @@ class GPIO(object):
 
     @property
     def chip_label(self):
-        """ Get the label of the GPIO chip associated with the GPIO.
+        """Get the label of the GPIO chip associated with the GPIO.
 
         :type: str
         """
@@ -224,6 +300,62 @@ class GPIO(object):
     :type: str
     """
 
+    def _get_bias(self):
+        raise NotImplementedError()
+
+    def _set_bias(self, bias):
+        raise NotImplementedError()
+
+    bias = property(_get_bias, _set_bias)
+    """Get or set the GPIO's line bias. Can be "default", "pull_up",
+    "pull_down", "disable".
+
+    This property is not supported by sysfs GPIOs.
+
+    Raises:
+        GPIOError: if an I/O or OS error occurs.
+        TypeError: if `bias` type is not str.
+        ValueError: if `bias` value is invalid.
+
+    :type: str
+    """
+
+    def _get_drive(self):
+        raise NotImplementedError()
+
+    def _set_drive(self, drive):
+        raise NotImplementedError()
+
+    drive = property(_get_drive, _set_drive)
+    """Get or set the GPIO's line drive. Can be "default" (for push-pull),
+    "open_drain", "open_source".
+
+    This property is not supported by sysfs GPIOs.
+
+    Raises:
+        GPIOError: if an I/O or OS error occurs.
+        TypeError: if `drive` type is not str.
+        ValueError: if `drive` value is invalid.
+
+    :type: str
+    """
+
+    def _get_inverted(self):
+        raise NotImplementedError()
+
+    def _set_inverted(self, inverted):
+        raise NotImplementedError()
+
+    inverted = property(_get_inverted, _set_inverted)
+    """Get or set the GPIO's inverted (active low) property.
+
+    Raises:
+        GPIOError: if an I/O or OS error occurs.
+        TypeError: if `inverted` type is not bool.
+
+    :type: bool
+    """
+
     # String representation
 
     def __str__(self):
@@ -295,61 +427,104 @@ class CdevGPIO(GPIO):
     _GPIO_GET_LINEEVENT_IOCTL = 0xc030b404
     _GPIOHANDLE_REQUEST_INPUT = 0x1
     _GPIOHANDLE_REQUEST_OUTPUT = 0x2
+    _GPIOHANDLE_REQUEST_ACTIVE_LOW = 0x4
+    _GPIOHANDLE_REQUEST_OPEN_DRAIN = 0x8
+    _GPIOHANDLE_REQUEST_OPEN_SOURCE = 0x10
+    _GPIOHANDLE_REQUEST_BIAS_PULL_UP = 0x20
+    _GPIOHANDLE_REQUEST_BIAS_PULL_DOWN = 0x40
+    _GPIOHANDLE_REQUEST_BIAS_DISABLE = 0x80
     _GPIOEVENT_REQUEST_RISING_EDGE = 0x1
     _GPIOEVENT_REQUEST_FALLING_EDGE = 0x2
     _GPIOEVENT_REQUEST_BOTH_EDGES = 0x3
     _GPIOEVENT_EVENT_RISING_EDGE = 0x1
     _GPIOEVENT_EVENT_FALLING_EDGE = 0x2
 
-    def __init__(self, path, line, direction):
+    def __init__(self, path, line, direction, edge="none", bias="default", drive="default", inverted=False, label=None):
         """**Character device GPIO**
 
         Instantiate a GPIO object and open the character device GPIO with the
         specified line and direction at the specified GPIO chip path (e.g.
-        "/dev/gpiochip0").
-
-        `direction` can be "in" for input; "out" for output, initialized to
-        low; "high" for output, initialized to high; or "low" for output,
-        initialized to low.
+        "/dev/gpiochip0"). Defaults properties can be overridden with keyword
+        arguments.
 
         Args:
             path (str): GPIO chip character device path.
             line (int, str): GPIO line number or name.
             direction (str): GPIO direction, can be "in", "out", "high", or
                              "low".
+            edge (str): GPIO interrupt edge, can be "none", "rising",
+                        "falling", or "both".
+            bias (str): GPIO line bias, can be "default", "pull_up",
+                        "pull_down", or "disable".
+            drive (str): GPIO line drive, can be "default", "open_drain", or
+                         "open_source".
+            inverted (bool): GPIO is inverted (active low).
+            label (str, None): GPIO line consumer label.
 
         Returns:
             CdevGPIO: GPIO object.
 
         Raises:
             GPIOError: if an I/O or OS error occurs.
-            TypeError: if `path`, `line`, or `direction`  types are invalid.
-            ValueError: if `direction` value is invalid.
+            TypeError: if `path`, `line`, `direction`, `edge`, `bias`, `drive`,
+                       `inverted`, or `label` types are invalid.
+            ValueError: if `direction`, `edge`, `bias`, or `drive` value is
+                        invalid.
             LookupError: if the GPIO line was not found by the provided name.
 
         """
         self._devpath = None
+        self._line = None
         self._line_fd = None
         self._chip_fd = None
-        self._edge = "none"
-        self._direction = "in"
-        self._line = None
+        self._direction = None
+        self._edge = None
+        self._bias = None
+        self._drive = None
+        self._inverted = None
+        self._label = None
 
-        self._open(path, line, direction)
+        self._open(path, line, direction, edge, bias, drive, inverted, label)
 
-    def __new__(self, path, line, direction):
+    def __new__(self, path, line, direction, **kwargs):
         return object.__new__(CdevGPIO)
 
-    def _open(self, path, line, direction):
+    def _open(self, path, line, direction, edge, bias, drive, inverted, label):
         if not isinstance(path, str):
             raise TypeError("Invalid path type, should be string.")
+
         if not isinstance(line, (int, str)):
             raise TypeError("Invalid line type, should be integer or string.")
+
         if not isinstance(direction, str):
             raise TypeError("Invalid direction type, should be string.")
-        if direction.lower() not in ["in", "out", "high", "low"]:
+        elif direction.lower() not in ["in", "out", "high", "low"]:
             raise ValueError("Invalid direction, can be: \"in\", \"out\", \"high\", \"low\".")
 
+        if not isinstance(edge, str):
+            raise TypeError("Invalid edge type, should be string.")
+        elif edge.lower() not in ["none", "rising", "falling", "both"]:
+            raise ValueError("Invalid edge, can be: \"none\", \"rising\", \"falling\", \"both\".")
+
+        if not isinstance(bias, str):
+            raise TypeError("Invalid bias type, should be string.")
+        elif bias.lower() not in ["default", "pull_up", "pull_down", "disable"]:
+            raise ValueError("Invalid bias, can be: \"default\", \"pull_up\", \"pull_down\", \"disable\".")
+
+        if not isinstance(drive, str):
+            raise TypeError("Invalid drive type, should be string.")
+        elif drive.lower() not in ["default", "open_drain", "open_source"]:
+            raise ValueError("Invalid drive, can be: \"default\", \"open_drain\", \"open_source\".")
+
+        if not isinstance(inverted, bool):
+            raise TypeError("Invalid drive type, should be bool.")
+
+        if not isinstance(label, (type(None), str)):
+            raise TypeError("Invalid label type, should be None or str.")
+
+        if isinstance(line, str):
+            line = self._find_line_by_name(path, line)
+
         # Open GPIO chip
         try:
             self._chip_fd = os.open(path, 0)
@@ -357,15 +532,33 @@ class CdevGPIO(GPIO):
             raise GPIOError(e.errno, "Opening GPIO chip: " + e.strerror)
 
         self._devpath = path
+        self._line = line
+        self._label = label.encode() if label is not None else b"periphery"
 
-        if isinstance(line, int):
-            self._line = line
-            self._reopen(direction, "none")
-        else:
-            self._line = self._find_line_by_name(line)
-            self._reopen(direction, "none")
+        self._reopen(direction, edge, bias, drive, inverted)
+
+    def _reopen(self, direction, edge, bias, drive, inverted):
+        flags = 0
+
+        if bias == "pull_up":
+            flags |= CdevGPIO._GPIOHANDLE_REQUEST_BIAS_PULL_UP
+        elif bias == "pull_down":
+            flags |= CdevGPIO._GPIOHANDLE_REQUEST_BIAS_PULL_DOWN
+        elif bias == "disable":
+            flags |= CdevGPIO._GPIOHANDLE_REQUEST_BIAS_DISABLE
+
+        if drive == "open_drain":
+            flags |= CdevGPIO._GPIOHANDLE_REQUEST_OPEN_DRAIN
+        elif drive == "open_source":
+            flags |= CdevGPIO._GPIOHANDLE_REQUEST_OPEN_SOURCE
+
+        if inverted:
+            flags |= CdevGPIO._GPIOHANDLE_REQUEST_ACTIVE_LOW
+
+        # FIXME this should really use GPIOHANDLE_SET_CONFIG_IOCTL instead of
+        # closing and reopening, especially to preserve output value on
+        # configuration changes
 
-    def _reopen(self, direction, edge):
         # Close existing line
         if self._line_fd is not None:
             try:
@@ -373,13 +566,15 @@ class CdevGPIO(GPIO):
             except OSError as e:
                 raise GPIOError(e.errno, "Closing existing GPIO line: " + e.strerror)
 
+            self._line_fd = None
+
         if direction == "in":
             if edge == "none":
                 request = _CGpiohandleRequest()
 
                 request.lineoffsets[0] = self._line
-                request.flags = CdevGPIO._GPIOHANDLE_REQUEST_INPUT
-                request.consumer_label = b"periphery"
+                request.flags = flags | CdevGPIO._GPIOHANDLE_REQUEST_INPUT
+                request.consumer_label = self._label
                 request.lines = 1
 
                 try:
@@ -388,15 +583,13 @@ class CdevGPIO(GPIO):
                     raise GPIOError(e.errno, "Opening input line handle: " + e.strerror)
 
                 self._line_fd = request.fd
-                self._direction = "in"
-                self._edge = "none"
             else:
                 request = _CGpioeventRequest()
 
                 request.lineoffset = self._line
-                request.handleflags = CdevGPIO._GPIOHANDLE_REQUEST_INPUT
+                request.handleflags = flags | CdevGPIO._GPIOHANDLE_REQUEST_INPUT
                 request.eventflags = CdevGPIO._GPIOEVENT_REQUEST_RISING_EDGE if edge == "rising" else CdevGPIO._GPIOEVENT_REQUEST_FALLING_EDGE if edge == "falling" else CdevGPIO._GPIOEVENT_REQUEST_BOTH_EDGES
-                request.consumer_label = b"periphery"
+                request.consumer_label = self._label
 
                 try:
                     fcntl.ioctl(self._chip_fd, CdevGPIO._GPIO_GET_LINEEVENT_IOCTL, request)
@@ -404,16 +597,15 @@ class CdevGPIO(GPIO):
                     raise GPIOError(e.errno, "Opening input line event handle: " + e.strerror)
 
                 self._line_fd = request.fd
-                self._direction = "in"
-                self._edge = edge
         else:
             request = _CGpiohandleRequest()
             initial_value = True if direction == "high" else False
+            initial_value ^= inverted
 
             request.lineoffsets[0] = self._line
-            request.flags = CdevGPIO._GPIOHANDLE_REQUEST_OUTPUT
+            request.flags = flags | CdevGPIO._GPIOHANDLE_REQUEST_OUTPUT
             request.default_values[0] = initial_value
-            request.consumer_label = b"periphery"
+            request.consumer_label = self._label
             request.lines = 1
 
             try:
@@ -422,28 +614,48 @@ class CdevGPIO(GPIO):
                 raise GPIOError(e.errno, "Opening output line handle: " + e.strerror)
 
             self._line_fd = request.fd
-            self._direction = "out"
-            self._edge = "none"
 
-    def _find_line_by_name(self, line):
+        self._direction = "in" if direction == "in" else "out"
+        self._edge = edge
+        self._bias = bias
+        self._drive = drive
+        self._inverted = inverted
+
+    def _find_line_by_name(self, path, line):
+        # Open GPIO chip
+        try:
+            fd = os.open(path, 0)
+        except OSError as e:
+            raise GPIOError(e.errno, "Opening GPIO chip: " + e.strerror)
+
         # Get chip info for number of lines
         chip_info = _CGpiochipInfo()
         try:
-            fcntl.ioctl(self._chip_fd, CdevGPIO._GPIO_GET_CHIPINFO_IOCTL, chip_info)
+            fcntl.ioctl(fd, CdevGPIO._GPIO_GET_CHIPINFO_IOCTL, chip_info)
         except (OSError, IOError) as e:
             raise GPIOError(e.errno, "Querying GPIO chip info: " + e.strerror)
 
         # Get each line info
         line_info = _CGpiolineInfo()
+        found = False
         for i in range(chip_info.lines):
             line_info.line_offset = i
             try:
-                fcntl.ioctl(self._chip_fd, CdevGPIO._GPIO_GET_LINEINFO_IOCTL, line_info)
+                fcntl.ioctl(fd, CdevGPIO._GPIO_GET_LINEINFO_IOCTL, line_info)
             except (OSError, IOError) as e:
                 raise GPIOError(e.errno, "Querying GPIO line info: " + e.strerror)
 
             if line_info.name.decode() == line:
-                return i
+                found = True
+                break
+
+        try:
+            os.close(fd)
+        except OSError as e:
+            raise GPIOError(e.errno, "Closing GPIO chip: " + e.strerror)
+
+        if found:
+            return i
 
         raise LookupError("Opening GPIO line: GPIO line \"{:s}\" not found by name.".format(line))
 
@@ -556,6 +768,18 @@ class CdevGPIO(GPIO):
 
         return line_info.name.decode()
 
+    @property
+    def label(self):
+        line_info = _CGpiolineInfo()
+        line_info.line_offset = self._line
+
+        try:
+            fcntl.ioctl(self._chip_fd, CdevGPIO._GPIO_GET_LINEINFO_IOCTL, line_info)
+        except (OSError, IOError) as e:
+            raise GPIOError(e.errno, "Querying GPIO line info: " + e.strerror)
+
+        return line_info.consumer.decode()
+
     @property
     def chip_fd(self):
         return self._chip_fd
@@ -596,7 +820,7 @@ class CdevGPIO(GPIO):
         if self._direction == direction:
             return
 
-        self._reopen(direction, "none")
+        self._reopen(direction, "none", self._bias, self._drive, self._inverted)
 
     direction = property(_get_direction, _set_direction)
 
@@ -615,10 +839,59 @@ class CdevGPIO(GPIO):
         if self._edge == edge:
             return
 
-        self._reopen("in", edge)
+        self._reopen(self._direction, edge, self._bias, self._drive, self._inverted)
 
     edge = property(_get_edge, _set_edge)
 
+    def _get_bias(self):
+        return self._bias
+
+    def _set_bias(self, bias):
+        if not isinstance(bias, str):
+            raise TypeError("Invalid bias type, should be string.")
+        if bias.lower() not in ["default", "pull_up", "pull_down", "disable"]:
+            raise ValueError("Invalid bias, can be: \"default\", \"pull_up\", \"pull_down\", \"disable\".")
+
+        if self._bias == bias:
+            return
+
+        self._reopen(self._direction, self._edge, bias, self._drive, self._inverted)
+
+    bias = property(_get_bias, _set_bias)
+
+    def _get_drive(self):
+        return self._drive
+
+    def _set_drive(self, drive):
+        if not isinstance(drive, str):
+            raise TypeError("Invalid drive type, should be string.")
+        if drive.lower() not in ["default", "open_drain", "open_source"]:
+            raise ValueError("Invalid drive, can be: \"default\", \"open_drain\", \"open_source\".")
+
+        if self._direction != "out" and drive != "default":
+            raise GPIOError(None, "Invalid operation: cannot set line drive on input GPIO")
+
+        if self._drive == drive:
+            return
+
+        self._reopen(self._direction, self._edge, self._bias, drive, self._inverted)
+
+    drive = property(_get_drive, _set_drive)
+
+    def _get_inverted(self):
+        return self._inverted
+
+    def _set_inverted(self, inverted):
+        if not isinstance(inverted, bool):
+            raise TypeError("Invalid drive type, should be bool.")
+
+        if self._inverted == inverted:
+            return
+
+        self._reopen(self._direction, self._edge, self._bias, self._drive, inverted)
+
+    inverted = property(_get_inverted, _set_inverted)
+
     # String representation
 
     def __str__(self):
@@ -627,6 +900,11 @@ class CdevGPIO(GPIO):
         except GPIOError:
             str_name = "<error>"
 
+        try:
+            str_label = self.label
+        except GPIOError:
+            str_label = "<error>"
+
         try:
             str_direction = self.direction
         except GPIOError:
@@ -637,6 +915,21 @@ class CdevGPIO(GPIO):
         except GPIOError:
             str_edge = "<error>"
 
+        try:
+            str_bias = self.bias
+        except GPIOError:
+            str_bias = "<error>"
+
+        try:
+            str_drive = self.drive
+        except GPIOError:
+            str_drive = "<error>"
+
+        try:
+            str_inverted = str(self.inverted)
+        except GPIOError:
+            str_inverted = "<error>"
+
         try:
             str_chip_name = self.chip_name
         except GPIOError:
@@ -647,8 +940,8 @@ class CdevGPIO(GPIO):
         except GPIOError:
             str_chip_label = "<error>"
 
-        return "GPIO {:d} (name=\"{:s}\", device={:s}, line_fd={:d}, chip_fd={:d}, direction={:s}, edge={:s}, chip_name=\"{:s}\", chip_label=\"{:s}\", type=cdev)" \
-            .format(self._line, str_name, self._devpath, self._line_fd, self._chip_fd, str_direction, str_edge, str_chip_name, str_chip_label)
+        return "GPIO {:d} (name=\"{:s}\", label=\"{:s}\", device={:s}, line_fd={:d}, chip_fd={:d}, direction={:s}, edge={:s}, bias={:s}, drive={:s}, inverted={:s}, chip_name=\"{:s}\", chip_label=\"{:s}\", type=cdev)" \
+            .format(self._line, str_name, str_label, self._devpath, self._line_fd, self._chip_fd, str_direction, str_edge, str_bias, str_drive, str_inverted, str_chip_name, str_chip_label)
 
 
 class SysfsGPIO(GPIO):
@@ -684,6 +977,7 @@ class SysfsGPIO(GPIO):
         """
         self._fd = None
         self._line = None
+        self._exported = False
 
         self._open(line, direction)
 
@@ -709,15 +1003,14 @@ class SysfsGPIO(GPIO):
                 raise GPIOError(e.errno, "Exporting GPIO: " + e.strerror)
 
             # Loop until GPIO is exported
-            exported = False
             for i in range(SysfsGPIO.GPIO_OPEN_RETRIES):
                 if os.path.isdir(gpio_path):
-                    exported = True
+                    self._exported = True
                     break
 
                 time.sleep(SysfsGPIO.GPIO_OPEN_DELAY)
 
-            if not exported:
+            if not self._exported:
                 raise TimeoutError("Exporting GPIO: waiting for \"{:s}\" timed out".format(gpio_path))
 
             # Write direction, looping in case of EACCES errors due to delayed udev
@@ -732,13 +1025,6 @@ class SysfsGPIO(GPIO):
                         raise GPIOError(e.errno, "Setting GPIO direction: " + e.strerror)
 
                 time.sleep(SysfsGPIO.GPIO_OPEN_DELAY)
-        else:
-            # Write direction
-            try:
-                with open(os.path.join(gpio_path, "direction"), "w") as f_direction:
-                    f_direction.write(direction.lower() + "\n")
-            except IOError as e:
-                raise GPIOError(e.errno, "Setting GPIO direction: " + e.strerror)
 
         # Open value
         try:
@@ -749,6 +1035,13 @@ class SysfsGPIO(GPIO):
         self._line = line
         self._path = gpio_path
 
+        # Initialize direction
+        if not self._exported:
+            self.direction = direction
+
+        # Initialize inverted
+        self.inverted = False
+
     # Methods
 
     def read(self):
@@ -831,13 +1124,14 @@ class SysfsGPIO(GPIO):
 
         self._fd = None
 
-        # Unexport the line
-        try:
-            unexport_fd = os.open("/sys/class/gpio/unexport", os.O_WRONLY)
-            os.write(unexport_fd, "{:d}\n".format(self._line).encode())
-            os.close(unexport_fd)
-        except OSError as e:
-            raise GPIOError(e.errno, "Unexporting GPIO: " + e.strerror)
+        if self._exported:
+            # Unexport the line
+            try:
+                unexport_fd = os.open("/sys/class/gpio/unexport", os.O_WRONLY)
+                os.write(unexport_fd, "{:d}\n".format(self._line).encode())
+                os.close(unexport_fd)
+            except OSError as e:
+                raise GPIOError(e.errno, "Unexporting GPIO: " + e.strerror)
 
     # Immutable properties
 
@@ -857,6 +1151,10 @@ class SysfsGPIO(GPIO):
     def name(self):
         return ""
 
+    @property
+    def label(self):
+        return ""
+
     @property
     def chip_fd(self):
         raise NotImplementedError("Sysfs GPIO does not have a gpiochip file descriptor.")
@@ -939,6 +1237,50 @@ class SysfsGPIO(GPIO):
 
     edge = property(_get_edge, _set_edge)
 
+    def _get_bias(self):
+        raise NotImplementedError("Sysfs GPIO does not support line bias property.")
+
+    def _set_bias(self, bias):
+        raise NotImplementedError("Sysfs GPIO does not support line bias property.")
+
+    bias = property(_get_bias, _set_bias)
+
+    def _get_drive(self):
+        raise NotImplementedError("Sysfs GPIO does not support line drive property.")
+
+    def _set_drive(self, drive):
+        raise NotImplementedError("Sysfs GPIO does not support line drive property.")
+
+    drive = property(_get_drive, _set_drive)
+
+    def _get_inverted(self):
+        # Read active_low
+        try:
+            with open(os.path.join(self._path, "active_low"), "r") as f_inverted:
+                inverted = f_inverted.read().strip()
+        except IOError as e:
+            raise GPIOError(e.errno, "Getting GPIO active_low: " + e.strerror)
+
+        if inverted == "0":
+            return False
+        elif inverted == "1":
+            return True
+
+        raise GPIOError(None, "Unknown GPIO active_low value: {}".format(inverted))
+
+    def _set_inverted(self, inverted):
+        if not isinstance(inverted, bool):
+            raise TypeError("Invalid drive type, should be bool.")
+
+        # Write active_low
+        try:
+            with open(os.path.join(self._path, "active_low"), "w") as f_active_low:
+                f_active_low.write("1\n" if inverted else "0\n")
+        except IOError as e:
+            raise GPIOError(e.errno, "Setting GPIO active_low: " + e.strerror)
+
+    inverted = property(_get_inverted, _set_inverted)
+
     # String representation
 
     def __str__(self):
@@ -962,5 +1304,10 @@ class SysfsGPIO(GPIO):
         except GPIOError:
             str_chip_label = "<error>"
 
-        return "GPIO {:d} (device={:s}, fd={:d}, direction={:s}, edge={:s}, chip_name=\"{:s}\", chip_label=\"{:s}\", type=sysfs)" \
-            .format(self._line, self._path, self._fd, str_direction, str_edge, str_chip_name, str_chip_label)
+        try:
+            str_inverted = str(self.inverted)
+        except GPIOError:
+            str_inverted = "<error>"
+
+        return "GPIO {:d} (device={:s}, fd={:d}, direction={:s}, edge={:s}, inverted={:s}, chip_name=\"{:s}\", chip_label=\"{:s}\", type=sysfs)" \
+            .format(self._line, self._path, self._fd, str_direction, str_edge, str_inverted, str_chip_name, str_chip_label)
