@@ -3,10 +3,12 @@ import ctypes
 import array
 import fcntl
 
+
 class I2CError(IOError):
     """Base class for I2C errors."""
     pass
 
+
 class _CI2CMessage(ctypes.Structure):
     _fields_ = [
         ("addr", ctypes.c_ushort),
@@ -15,25 +17,27 @@ class _CI2CMessage(ctypes.Structure):
         ("buf", ctypes.POINTER(ctypes.c_ubyte)),
     ]
 
+
 class _CI2CIocTransfer(ctypes.Structure):
     _fields_ = [
         ("msgs", ctypes.POINTER(_CI2CMessage)),
         ("nmsgs", ctypes.c_uint),
     ]
 
+
 class I2C(object):
     # Constants scraped from <linux/i2c-dev.h> and <linux/i2c.h>
-    _I2C_IOC_FUNCS      = 0x705
-    _I2C_IOC_RDWR       = 0x707
-    _I2C_FUNC_I2C       = 0x1
-    _I2C_M_TEN          = 0x0010
-    _I2C_M_RD           = 0x0001
-    _I2C_M_STOP         = 0x8000
-    _I2C_M_NOSTART      = 0x4000
+    _I2C_IOC_FUNCS = 0x705
+    _I2C_IOC_RDWR = 0x707
+    _I2C_FUNC_I2C = 0x1
+    _I2C_M_TEN = 0x0010
+    _I2C_M_RD = 0x0001
+    _I2C_M_STOP = 0x8000
+    _I2C_M_NOSTART = 0x4000
     _I2C_M_REV_DIR_ADDR = 0x2000
-    _I2C_M_IGNORE_NAK   = 0x1000
-    _I2C_M_NO_RD_ACK    = 0x0800
-    _I2C_M_RECV_LEN     = 0x0400
+    _I2C_M_IGNORE_NAK = 0x1000
+    _I2C_M_NO_RD_ACK = 0x0800
+    _I2C_M_RECV_LEN = 0x0400
 
     def __init__(self, devpath):
         """Instantiate an I2C object and open the i2c-dev device at the
@@ -57,7 +61,7 @@ class I2C(object):
         self.close()
 
     def __enter__(self):
-        pass
+        return self
 
     def __exit__(self, t, value, traceback):
         self.close()
@@ -75,14 +79,14 @@ class I2C(object):
         buf = array.array('I', [0])
         try:
             fcntl.ioctl(self._fd, I2C._I2C_IOC_FUNCS, buf, True)
-        except OSError as e:
+        except (OSError, IOError) as e:
             self.close()
             raise I2CError(e.errno, "Querying supported functions: " + e.strerror)
 
         # Check that I2C_RDWR ioctl() is supported on this device
         if (buf[0] & I2C._I2C_FUNC_I2C) == 0:
             self.close()
-            raise I2CError(None, "I2C not supported on device %s." % devpath)
+            raise I2CError(None, "I2C not supported on device \"{:s}\"".format(devpath))
 
     # Methods
 
@@ -129,7 +133,7 @@ class I2C(object):
         # Transfer
         try:
             fcntl.ioctl(self._fd, I2C._I2C_IOC_RDWR, i2c_xfer, False)
-        except IOError as e:
+        except (OSError, IOError) as e:
             raise I2CError(e.errno, "I2C transfer: " + e.strerror)
 
         # Update any read I2C.Message messages
@@ -182,7 +186,7 @@ class I2C(object):
     # String representation
 
     def __str__(self):
-        return "I2C (device=%s, fd=%d)" % (self.devpath, self.fd)
+        return "I2C (device={:s}, fd={:d})".format(self.devpath, self.fd)
 
     class Message:
         def __init__(self, data, read=False, flags=0):
@@ -212,4 +216,3 @@ class I2C(object):
             self.data = data
             self.read = read
             self.flags = flags
-
