@@ -4,10 +4,12 @@ import array
 import termios
 import select
 
+
 class SerialError(IOError):
     """Base class for Serial errors."""
     pass
 
+
 class Serial(object):
     _DATABITS_TO_CFLAG = {
         5: termios.CS5, 6: termios.CS6, 7: termios.CS7, 8: termios.CS8
@@ -55,13 +57,14 @@ class Serial(object):
         """
         self._fd = None
         self._devpath = None
+        self._use_termios_timeout = False
         self._open(devpath, baudrate, databits, parity, stopbits, xonxoff, rtscts)
 
     def __del__(self):
         self.close()
 
     def __enter__(self):
-        pass
+        return self
 
     def __exit__(self, t, value, traceback):
         self.close()
@@ -83,7 +86,7 @@ class Serial(object):
             raise TypeError("Invalid rtscts type, should be boolean.")
 
         if baudrate not in Serial._BAUDRATE_TO_OSPEED:
-            raise ValueError("Unknown baud rate %d." % baudrate)
+            raise ValueError("Unknown baud rate: {:d}".format(baudrate))
         elif databits not in [5, 6, 7, 8]:
             raise ValueError("Invalid data bits, can be 5, 6, 7, 8.")
         elif parity.lower() not in ["none", "even", "odd"]:
@@ -101,9 +104,10 @@ class Serial(object):
 
         parity = parity.lower()
 
-        (iflag, oflag, cflag, lflag, ispeed, ospeed, cc) = (0, 0, 0, 0, 0, 0, [0]*32)
+        (iflag, oflag, cflag, lflag, ispeed, ospeed, cc) = (0, 0, 0, 0, 0, 0, [0] * 32)
 
-        ### iflag
+        ###
+        # iflag
 
         # Ignore break characters
         iflag = termios.IGNBRK
@@ -116,13 +120,16 @@ class Serial(object):
         if xonxoff:
             iflag |= (termios.IXON | termios.IXOFF)
 
-        ### oflag
+        #######
+        # oflag
         oflag = 0
 
-        ### lflag
+        #######
+        # lflag
         lflag = 0
 
-        ### cflag
+        #######
+        # cflag
 
         # Enable receiver, ignore modem control lines
         cflag = (termios.CREAD | termios.CLOCAL)
@@ -147,10 +154,12 @@ class Serial(object):
         # Setup baud rate
         cflag |= Serial._BAUDRATE_TO_OSPEED[baudrate]
 
-        ### ispeed
+        ########
+        # ispeed
         ispeed = Serial._BAUDRATE_TO_OSPEED[baudrate]
 
-        ### ospeed
+        ########
+        # ospeed
         ospeed = Serial._BAUDRATE_TO_OSPEED[baudrate]
 
         # Set tty attributes
@@ -159,18 +168,28 @@ class Serial(object):
         except termios.error as e:
             raise SerialError(e.errno, "Setting serial port attributes: " + e.strerror)
 
+        self._use_termios_timeout = False
+
     # Methods
 
     def read(self, length, timeout=None):
         """Read up to `length` number of bytes from the serial port with an
         optional timeout.
 
-        `timeout` can be positive for a timeout in seconds, 0 for a
-        non-blocking read, or negative or None for a blocking read that will
-        block until `length` number of bytes are read.
+        `timeout` can be positive for a blocking read with a timeout in
+        seconds, zero for a non-blocking read, or negative or None for a
+        blocking read that will block until `length` number of bytes are read.
+        Default is a blocking read.
 
-        For a non-blocking or timeout bound read, read() may return data whose
-        length is less than or equal to the requested length.
+        For a non-blocking or timeout-bound read, `read()` may return less than
+        the requested number of bytes.
+
+        For a blocking read with the VMIN setting configured, `read()` will
+        block until at least VMIN bytes are read. For a blocking read with both
+        VMIN and VTIME settings configured, `read()` will block until at least
+        VMIN bytes are read or the VTIME interbyte timeout expires after the
+        last byte read. In either case, `read()` may return less than the
+        requested number of bytes.
 
         Args:
             length (int): length in bytes.
@@ -185,24 +204,23 @@ class Serial(object):
         """
         data = b""
 
-        # Read length bytes if timeout is None
-
-        # Read up to length bytes if timeout is not None
-        while True:
-            if timeout is not None:
-                # Select
-                (rlist, _, _) = select.select([self._fd], [], [], timeout)
-                # If timeout
-                if self._fd not in rlist:
-                    break
+        while len(data) < length:
+            (rlist, _, _) = select.select([self._fd], [], [], timeout)
+            if self._fd not in rlist:
+                break
 
             try:
-                data += os.read(self._fd, length-len(data))
+                chunk = os.read(self._fd, length - len(data))
             except OSError as e:
                 raise SerialError(e.errno, "Reading serial port: " + e.strerror)
 
-            if len(data) == length:
-                break
+            if self._use_termios_timeout:
+                return chunk
+
+            if not chunk:
+                raise SerialError(0, "Reading serial port: unexpected empty read")
+
+            data += chunk
 
         return data
 
@@ -222,7 +240,7 @@ class Serial(object):
             ValueError: if data is not valid bytes.
 
         """
-        if not isinstance(data, bytes) and not isinstance(data, bytearray) and not isinstance(data, list):
+        if not isinstance(data, (bytes, bytearray, list)):
             raise TypeError("Invalid data type, should be bytes, bytearray, or list.")
 
         if isinstance(data, list):
@@ -233,11 +251,13 @@ class Serial(object):
         except OSError as e:
             raise SerialError(e.errno, "Writing serial port: " + e.strerror)
 
-    def poll(self, timeout):
-        """Poll for data available for reading from the serial port.
+    def poll(self, timeout=None):
+        """Poll for data available for reading from the serial port with an
+        optional timeout.
 
-        `timeout` can be positive for a timeout in seconds, 0 for a
-        non-blocking poll, or negative or None for a blocking poll.
+        `timeout` can be positive for a timeout in seconds, zero for a
+        non-blocking poll, or negative or None for a blocking poll. Default is
+        a blocking poll.
 
         Args:
             timeout (int, float, None): timeout duration in seconds.
@@ -248,7 +268,7 @@ class Serial(object):
         """
         p = select.poll()
         p.register(self._fd, select.POLLIN | select.POLLPRI)
-        events = p.poll(int(timeout*1000))
+        events = p.poll(int(timeout * 1000))
 
         if len(events) > 0:
             return True
@@ -282,12 +302,12 @@ class Serial(object):
         buf = array.array('I', [0])
         try:
             fcntl.ioctl(self._fd, termios.TIOCINQ, buf, True)
-        except OSError as e:
+        except (OSError, IOError) as e:
             raise SerialError(e.errno, "Querying input waiting: " + e.strerror)
 
         return buf[0]
 
-    def output_wating(self):
+    def output_waiting(self):
         """Query the number of bytes waiting to be written to the serial port.
 
         Returns:
@@ -301,13 +321,13 @@ class Serial(object):
         buf = array.array('I', [0])
         try:
             fcntl.ioctl(self._fd, termios.TIOCOUTQ, buf, True)
-        except OSError as e:
+        except (OSError, IOError) as e:
             raise SerialError(e.errno, "Querying output waiting: " + e.strerror)
 
         return buf[0]
 
     def close(self):
-        """Close the tty Serial device.
+        """Close the tty device.
 
         Raises:
             SerialError: if an I/O or OS error occurs.
@@ -351,7 +371,7 @@ class Serial(object):
             raise SerialError(e.errno, "Getting serial port attributes: " + e.strerror)
 
         if ospeed not in Serial._OSPEED_TO_BAUDRATE:
-            raise SerialError(None, "Unknown baud rate: ospeed 0x%x." % ospeed)
+            raise SerialError(None, "Unknown baud rate: ospeed 0x{:x}".format(ospeed))
 
         return Serial._OSPEED_TO_BAUDRATE[ospeed]
 
@@ -360,7 +380,7 @@ class Serial(object):
             raise TypeError("Invalid baud rate type, should be integer.")
 
         if baudrate not in Serial._BAUDRATE_TO_OSPEED:
-            raise ValueError("Unknown baud rate %d." % baudrate)
+            raise ValueError("Unknown baud rate: {:d}".format(baudrate))
 
         # Get tty attributes
         try:
@@ -401,7 +421,7 @@ class Serial(object):
         cs = cflag & termios.CSIZE
 
         if cs not in Serial._CFLAG_TO_DATABITS:
-            raise SerialError(None, "Unknown data bits setting: csize 0x%x." % cs)
+            raise SerialError(None, "Unknown data bits setting: csize 0x{:x}".format(cs))
 
         return Serial._CFLAG_TO_DATABITS[cs]
 
@@ -411,7 +431,6 @@ class Serial(object):
         elif databits not in [5, 6, 7, 8]:
             raise ValueError("Invalid data bits, can be 5, 6, 7, 8.")
 
-
         # Get tty attributes
         try:
             (iflag, oflag, cflag, lflag, ispeed, ospeed, cc) = termios.tcgetattr(self._fd)
@@ -625,8 +644,96 @@ class Serial(object):
     :type: bool
     """
 
+    def _get_vmin(self):
+        try:
+            iflag, oflag, cflag, lflag, ispeed, ospeed, cc = termios.tcgetattr(self._fd)
+        except termios.error as e:
+            raise SerialError(e.errno, "Getting serial port attributes: " + e.strerror)
+
+        return cc[termios.VMIN]
+
+    def _set_vmin(self, vmin):
+        if not isinstance(vmin, int):
+            raise TypeError("Invalid vmin type, should be integer.")
+        elif not (0 <= vmin <= 255):
+            raise ValueError("Invalid vmin, can be 0 to 255.")
+
+        try:
+            iflag, oflag, cflag, lflag, ispeed, ospeed, cc = termios.tcgetattr(self._fd)
+        except termios.error as e:
+            raise SerialError(e.errno, "Setting serial port attributes: " + e.strerror)
+
+        cc[termios.VMIN] = vmin
+
+        try:
+            termios.tcsetattr(self._fd, termios.TCSANOW, [iflag, oflag, cflag, lflag, ispeed, ospeed, cc])
+        except termios.error as e:
+            raise SerialError(e.errno, "Setting serial port attributes: " + e.strerror)
+
+        self._use_termios_timeout = vmin > 0
+
+    vmin = property(_get_vmin, _set_vmin)
+    """Get or set the VMIN termios setting for minimum number of bytes returned
+    from a blocking read. Can be between 0 and 255.
+
+    When configured in conjunction with VTIME, VTIME acts as an interbyte
+    timeout that restarts on every byte received, and a blocking read will
+    block until at least VMIN bytes are read or the VTIME timeout expires after
+    the last byte read. See the `termios` man page for more information.
+
+    Raises:
+        SerialError: if an I/O or OS error occurs.
+        TypeError: if `vmin` type is not int.
+        ValueError: if `vmin` value is invalid.
+
+    :type: int
+    """
+
+    def _get_vtime(self):
+        try:
+            iflag, oflag, cflag, lflag, ispeed, ospeed, cc = termios.tcgetattr(self._fd)
+        except termios.error as e:
+            raise SerialError(e.errno, "Getting serial port attributes: " + e.strerror)
+
+        return float(cc[termios.VTIME]) / 10.0
+
+    def _set_vtime(self, vtime):
+        if not isinstance(vtime, (float, int)):
+            raise TypeError("Invalid vtime type, should be float or integer.")
+        elif not (0 <= vtime <= 25.5):
+            raise ValueError("Invalid vtime, can be 0 to 25.5 seconds.")
+
+        try:
+            iflag, oflag, cflag, lflag, ispeed, ospeed, cc = termios.tcgetattr(self._fd)
+        except termios.error as e:
+            raise SerialError(e.errno, "Setting serial port attributes: " + e.strerror)
+
+        cc[termios.VTIME] = int(float(vtime) * 10.0)
+
+        try:
+            termios.tcsetattr(self._fd, termios.TCSANOW, [iflag, oflag, cflag, lflag, ispeed, ospeed, cc])
+        except termios.error as e:
+            raise SerialError(e.errno, "Setting serial port attributes: " + e.strerror)
+
+    vtime = property(_get_vtime, _set_vtime)
+    """Get or set the VTIME termios setting for timeout in seconds of a
+    blocking read. Can be between 0 to 25.5 seconds, with a resolution of 0.1
+    seconds.
+
+    When configured in conjunction with VMIN, VTIME acts as an interbyte
+    timeout that restarts on every byte received, and a blocking read will
+    block until at least VMIN bytes are read or the VTIME timeout expires after
+    the last byte read. See the `termios` man page for more information.
+
+    Raises:
+        SerialError: if an I/O or OS error occurs.
+        TypeError: if `vtime` type is not float or int.
+        ValueError: if `vtime` value is invalid.
+
+    :type: float
+    """
+
     # String representation
-
     def __str__(self):
-        return "Serial (device=%s, fd=%d, baudrate=%d, databits=%d, parity=%s, stopbits=%d, xonxoff=%s, rtscts=%s)" % (self.devpath, self.fd, self.baudrate, self.databits, self.parity, self.stopbits, str(self.xonxoff), str(self.rtscts))
-
+        return "Serial (device={:s}, fd={:d}, baudrate={:d}, databits={:d}, parity={:s}, stopbits={:d}, xonxoff={:s}, rtscts={:s}, vmin={:d}, vtime={:.1f})" \
+            .format(self.devpath, self.fd, self.baudrate, self.databits, self.parity, self.stopbits, str(self.xonxoff), str(self.rtscts), self.vmin, self.vtime)
